@@ -104,7 +104,7 @@ impl<T: Copy + 'static, const N: usize> Vector<T> for StaticVec<T, N> {
   }
 
   fn from_elem(val: T, _n: usize) -> Self {
-    staticvec![val; {N}]
+    staticvec::utils::new_from_value(val)
   }
 
   fn from_elems(val: &[T]) -> Self {
